#!/usr/bin/env python
import gym
import safety_gym
import safe_rl
from safe_rl.utils.run_utils import setup_logger_kwargs
#from safe_rl.utils.mpi_tools import mpi_fork

def main(env_name, algo, seed, exp_name, cpu):

    # Verify experiment
    algo_list = ['rcpo', 'ppo_lagrangian', 'cpo']

    algo = algo.lower()
    assert algo in algo_list, "Invalid algo"


    # Hyperparameters
    exp_name = algo + '_' + env_name
    num_steps = 10e7
    steps_per_epoch = 30000
    epochs = int(num_steps / steps_per_epoch)
    save_freq = 50
    target_kl = 0.01
    cost_lim = 25

    # Fork for parallelizing
    #mpi_fork(cpu)

    # Prepare Logger
    exp_name = exp_name or (algo + '_' + env_name.lower())
    logger_kwargs = setup_logger_kwargs(exp_name, seed)

    # Algo and Env
    algo = eval('safe_rl.'+algo)
    env_name = env_name

    algo(env_fn=lambda: gym.make(env_name),
         ac_kwargs=dict(
             hidden_sizes=(256, 256),
            ),
         epochs=epochs,
         steps_per_epoch=steps_per_epoch,
         save_freq=save_freq,
         target_kl=target_kl,
         cost_lim=cost_lim,
         seed=seed,
         logger_kwargs=logger_kwargs
         )


if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('--env', type=str, default='CartSafe-v0')
    parser.add_argument('--algo', type=str, default='ppo_lagrangian')
    parser.add_argument('--seed', type=int, default=0)
    parser.add_argument('--exp_name', type=str, default='')
    parser.add_argument('--cpu', type=int, default=20)
    args = parser.parse_args()
    exp_name = args.exp_name if not(args.exp_name=='') else None
    main(args.env, args.algo, args.seed, exp_name, args.cpu)
# python scripts/experiment_safe.py --env Safexp-PointGoal1-v0 --algo ppo_lagrangian --seed 532 --exp_name ppo_point_button --cpu 14
